package net.sf.corn.gate;

import java.io.Serializable;
import java.util.Collections;
import java.util.List;
import java.util.Set;
import java.util.UUID;
import java.util.concurrent.ConcurrentSkipListSet;
import java.util.concurrent.CopyOnWriteArrayList;

/**
 * The default implementation {@link IServiceFilter} . 
 * @author Serhat Dirik
 */
public class ServiceFilter implements Serializable,IServiceFilter {

	private static final long serialVersionUID = 1929770869037188130L;
	private String name;
	private String description;
	private Set<ServiceTargetDefinition> serviceTargets = new ConcurrentSkipListSet<ServiceTargetDefinition>();
	private List<IExecutionAdvice> advices = new CopyOnWriteArrayList<>();

	/**
	 * Constructs a service filter with a random UUID name
	 */
	public ServiceFilter(){
		this(UUID.randomUUID().toString());
	}
	
	/**
	 * Constructs a filter with the given name
	 * @param name name of this service target
	 */
 	public ServiceFilter(String name) {
 		this.name = name;
	}
 	/**
 	 * Sets the service filter name
 	 * @param name name of the filter
 	 * @return filter object
 	 */
 	public ServiceFilter name(String name) {
		this.name = name;
		return this;
	}
 	/**
 	 * Sets the description of the filter
 	 * @param description a text explanation about the filter
 	 * @return filter object
 	 */
 	public ServiceFilter description(String description) {
		this.description = description;
		return this;
	}
 	/**
 	 * Adds a target to the filter
 	 * @param target target of this filter
 	 * @return filter object
 	 */
 	public ServiceFilter addServiceTarget(ServiceTargetDefinition target){
 		serviceTargets.add(target);
 		return this;
 	}
 	/**
 	 * Adds an advice to the filter
 	 * @param advice execution advice
 	 * @return filter object
 	 */
 	public ServiceFilter addAdvice(IExecutionAdvice advice){
 		if(! advices.contains(advice))	advices.add(advice);
 		return this;
 	}
 	
 	/**
 	 * Removes the given advice from the filter
 	 * @param advice advice object to be removed
 	 * @return true if successfully removed
 	 */
 	public boolean removeAdvice(IExecutionAdvice advice){
 		return advices.remove(advice);
 	}
 	
 	@Override
 	public List<IExecutionAdvice> getAdvice(){
 		return Collections.unmodifiableList(advices);
 	}
 	
 	@Override
 	public Set<ServiceTargetDefinition> getServiceTargetDefinitions(){
 		return Collections.unmodifiableSet(serviceTargets);
 	}
 	@Override
	public String getName() {
		return name;
	}
 	@Override
	public String getDescription(){
		return description;
	}
	
 	@Override
	public boolean matches(String serviceName,String operationName){
		for (ServiceTargetDefinition target : serviceTargets) {
			if(target.matches(serviceName, operationName)) return true;
		}
		return false;
	}

	@Override
	public String toString() {
		return "ServiceFilter [name=" + name + ", serviceTargets="
				+ serviceTargets + "]";
	}
 
	
}
